//+------------------------------------------------------------------+
//|                                                        Model.mqh |
//|                                Copyright 2021, Andrei Novichkov. |
//|                                              https://fxstill.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2021, Andrei Novichkov."
#property link      "https://fxstill.com"
#property version   "1.00"

#include "InputBase.mqh"
#include "IOutputBase.mqh"

class CModel {

 public:
   CModel();
   ~CModel();
   int Initialize(IInputBase* pI, IOutputBase* pO){
      pInput  = pI;
      pOutput = pO;
      iLength = pI.GetPeriod();
      return INIT_SUCCEEDED;
   }
   int Tick(const int rates_total,const int prev_calculated,const datetime &time[],const double &open[],const double &high[],const double &low[],const double &close[],const long &tick_volume[],const long &volume[],const int &spread[]);
protected:
   double Highest(const double &array[], int period, int cur_position);
   double Lowest(const double &array[], int period, int cur_position);
private:
   IInputBase*  pInput;
   IOutputBase* pOutput; 
   int          iLength;
};
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
CModel::CModel() {
   
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
CModel::~CModel() {
}
//+------------------------------------------------------------------+

int CModel::Tick(const int rates_total,const int prev_calculated,const datetime &time[],const double &open[],const double &high[],const double &low[],const double &close[],const long &tick_volume[],const long &volume[],const int &spread[]) {

   if(rates_total < iLength)
      return(0);

//--- start working
   int i, pOutputs = prev_calculated - 1;
   if(pOutputs < iLength - 1) {
      pOutputs = iLength - 1;
      for(i = 0; i < pOutputs; i++)
         pOutput.SetValue(i, 0);
   }

//---  main cycle
   double w;
   for(i = pOutputs; i < rates_total && !IsStopped(); i++) {
      double max_high = Highest(high, iLength,i);
      double min_low  = Lowest(low, iLength, i);
      //--- calculate WPR
      if(max_high != min_low) {
         w = -(max_high - close[i]) * 100 / (max_high - min_low);
         pOutput.SetValue(i, w);
      } else
            pOutput.SetValue(i, pOutput.GetValue(i - 1) ); 
   }
   
//--- return new prev_calculated value
   return(rates_total);

}

//+------------------------------------------------------------------+
//| Maximum High                                                     |
//+------------------------------------------------------------------+
double CModel::Highest(const double &array[], int period, int cur_position)
  {
   double res = array[cur_position];
   for(int i = cur_position - 1; i > cur_position - period && i >= 0; i--)
      if(res < array[i])
         res = array[i];
   return(res);
  }
//+------------------------------------------------------------------+
//| Minimum Low                                                      |
//+------------------------------------------------------------------+
double CModel::Lowest(const double &array[], int period, int cur_position)
  {
   double res = array[cur_position];
   for(int i = cur_position - 1; i > cur_position - period && i >= 0; i--)
      if(res > array[i])
         res = array[i];
   return(res);
  }
